@@ -4,6 +4,8 @@
 #define _USE_MATH_DEFINES
 #include <cmath>
 #include <algorithm>
+#include <R.h>
+#include <Rinternals.h>
 
 namespace ColorSpace {
 	double Hue_2_RGB(double v1, double v2, double vh) {
@@ -18,17 +20,32 @@ namespace ColorSpace {
 
 
 	void RgbConverter::ToColorSpace(Rgb *color, Rgb *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		item->r = color->r;
 		item->g = color->g;
 		item->b = color->b;
 	}
 	void RgbConverter::ToColor(Rgb *color, Rgb *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		color->r = item->r;
 		color->g = item->g;
 		color->b = item->b;
 	}
 
 	void XyzConverter::ToColorSpace(Rgb *color, Xyz *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		double r = color->r / 255.0;
 		double g = color->g / 255.0;
 		double b = color->b / 255.0;
@@ -42,6 +59,11 @@ namespace ColorSpace {
 		item->z = r*0.0193339 + g*0.1191920 + b*0.9503041;
 	}
 	void XyzConverter::ToColor(Rgb *color, Xyz *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		double x = item->x / 100.0;
 		double y = item->y / 100.0;
 		double z = item->z / 100.0;
@@ -61,8 +83,19 @@ namespace ColorSpace {
 	const double XyzConverter::eps = 216.0 / 24389.0;
 	const double XyzConverter::kappa = 24389.0 / 27.0;
 	Xyz XyzConverter::whiteReference(95.047, 100.000, 108.883); // D65
+	void XyzConverter::SetWhiteReference(double x, double y, double z) {
+	  return;
+	  whiteReference.x = x;
+	  whiteReference.y = y;
+	  whiteReference.z = z;
+	}
 
 	void HslConverter::ToColorSpace(Rgb *color, Hsl *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		double r = color->r / 255.0;
 		double g = color->g / 255.0;
 		double b = color->b / 255.0;
@@ -98,6 +131,11 @@ namespace ColorSpace {
 		item->l *= 100;
 	}
 	void HslConverter::ToColor(Rgb *color, Hsl *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		double h = item->h / 360;
 		double s = item->s / 100;
 		double l = item->l / 100;
@@ -118,6 +156,11 @@ namespace ColorSpace {
 	}
 
 	void LabConverter::ToColorSpace(Rgb *color, Lab *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 	  const Xyz &white = XyzConverter::whiteReference;
 		Xyz xyz;
 
@@ -136,6 +179,11 @@ namespace ColorSpace {
 		item->b = 200 * (y - z);
 	}
 	void LabConverter::ToColor(Rgb *color, Lab *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 	  const Xyz &white = XyzConverter::whiteReference;
 	  
 		double y = (item->l + 16.0) / 116.0;
@@ -155,6 +203,11 @@ namespace ColorSpace {
 	}
 
 	void LchConverter::ToColorSpace(Rgb *color, Lch *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		Lab lab;
 
 		LabConverter::ToColorSpace(color, &lab);
@@ -175,6 +228,11 @@ namespace ColorSpace {
 		item->h = h;
 	}
 	void LchConverter::ToColor(Rgb *color, Lch *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		Lab lab;
 
 		item->h = item->h * M_PI / 180;
@@ -187,6 +245,11 @@ namespace ColorSpace {
 	}
 
 	void LuvConverter::ToColorSpace(Rgb *color, Luv *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		const Xyz &white = XyzConverter::whiteReference;
 		Xyz xyz;
 
@@ -200,27 +263,47 @@ namespace ColorSpace {
 		item->v = 117 * item->l * (((temp > 1e-3) ? (xyz.y / temp) : 0) - white.y / tempr);
 	}
 	void LuvConverter::ToColor(Rgb *color, Luv *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		const Xyz &white = XyzConverter::whiteReference;
 		Xyz xyz;
-
-		double y = (item->l > XyzConverter::eps*XyzConverter::kappa) ? POW3((item->l + 16) / 116) : (item->l / XyzConverter::kappa);
-		double tempr = white.x + 15 * white.y + 3 * white.z;
-		double up = 4 * white.x / tempr;
-		double vp = 9 * white.y / tempr;
-
-		double a = 1. / 3. * (52 * item->l / (item->u + 13 * item->l*up) - 1);
-		double b = -5 * y;
-		double x = (y*(39 * item->l / (item->v + 13 * item->l*vp) - 5) - b) / (a + 1. / 3.);
-		double z = x*a + b;
-
-		xyz.x = x * 100;
-		xyz.y = y * 100;
-		xyz.z = z * 100;
+		if (item->l == 0.0) {
+		  xyz.x = 0.0;
+		  xyz.y = 0.0;
+		  xyz.z = 0.0;
+		} else {
+		  double y = (item->l > XyzConverter::eps*XyzConverter::kappa) ? POW3((item->l + 16) / 116) : (item->l / XyzConverter::kappa);
+		  double tempr = white.x + 15 * white.y + 3 * white.z;
+		  double up = 4 * white.x / tempr;
+		  double vp = 9 * white.y / tempr;
+		  double a = 1. / 3. * (52 * item->l / (item->u + 13 * item->l*up) - 1);
+		  double b = -5 * y;
+		  double x = (y*(39 * item->l / (item->v + 13 * item->l*vp) - 5) - b) / (a + 1. / 3.);
+		  double z = x*a + b;
+		  /* Weird things can happen at the edge of 0.0*/
+		  if (!ISNAN(z)) {
+		    xyz.x = x * 100;
+		    xyz.y = y * 100;
+		    xyz.z = z * 100;
+		  } else {
+		    xyz.x = 0.0;
+		    xyz.y = 0.0;
+		    xyz.z = 0.0;
+		  }
+		}
 
 		XyzConverter::ToColor(color, &xyz);
 	}
 
 	void YxyConverter::ToColorSpace(Rgb *color, Yxy *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		Xyz xyz;
 
 		XyzConverter::ToColorSpace(color, &xyz);
@@ -230,6 +313,11 @@ namespace ColorSpace {
 		item->y2 = (temp==0) ? 0 : (xyz.y / temp);
 	}
 	void YxyConverter::ToColor(Rgb *color, Yxy *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		Xyz xyz;
 
 		xyz.x = item->x*(item->y1 / item->y2);
@@ -239,17 +327,32 @@ namespace ColorSpace {
 	}
 
 	void CmyConverter::ToColorSpace(Rgb *color, Cmy *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		item->c = 1 - color->r / 255;
 		item->m = 1 - color->g / 255;
 		item->y = 1 - color->b / 255;
 	}
 	void CmyConverter::ToColor(Rgb *color, Cmy *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		color->r = (1 - item->c) * 255;
 		color->g = (1 - item->m) * 255;
 		color->b = (1 - item->y) * 255;
 	}
 
 	void CmykConverter::ToColorSpace(Rgb *color, Cmyk *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		Cmy cmy;
 
 		CmyConverter::ToColorSpace(color, &cmy);
@@ -271,6 +374,11 @@ namespace ColorSpace {
 		}
 	}
 	void CmykConverter::ToColor(Rgb *color, Cmyk *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		Cmy cmy;
 
 		cmy.c = item->c * (1 - item->k) + item->k;
@@ -280,6 +388,11 @@ namespace ColorSpace {
 	}
 
 	void HsvConverter::ToColorSpace(Rgb *color, Hsv *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		double r = color->r / 255.0;
 		double g = color->g / 255.0;
 		double b = color->b / 255.0;
@@ -310,6 +423,11 @@ namespace ColorSpace {
 		}
 	}
 	void HsvConverter::ToColor(Rgb *color, Hsv *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		int range = (int)std::floor(item->h / 60);
 		double c = item->v*item->s;
 		double x = c*(1 - std::abs(std::fmod(item->h / 60, 2) - 1));
@@ -350,6 +468,11 @@ namespace ColorSpace {
 	}
 
 	void HsbConverter::ToColorSpace(Rgb *color, Hsb *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		Hsv hsv;
 
 		HsvConverter::ToColorSpace(color, &hsv);
@@ -358,6 +481,11 @@ namespace ColorSpace {
 		item->b = hsv.v;
 	}
 	void HsbConverter::ToColor(Rgb *color, Hsb *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		Hsv hsv;
 
 		hsv.h = item->h;
@@ -367,6 +495,11 @@ namespace ColorSpace {
 	}
 
 	void HunterLabConverter::ToColorSpace(Rgb *color, HunterLab *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
 		Xyz xyz;
 
 		XyzConverter::ToColorSpace(color, &xyz);
@@ -375,6 +508,11 @@ namespace ColorSpace {
 		item->b = (xyz.y != 0) ? (7.0*(xyz.y - 0.847*xyz.z) / std::sqrt(xyz.y)) : 0;
 	}
 	void HunterLabConverter::ToColor(Rgb *color, HunterLab *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
 		double x = (item->a / 17.5) *(item->l / 10.0);
 		double y = item->l*item->l / 100;
 		double z = item->b / 7.0 * item->l / 10.0;
@@ -382,5 +520,47 @@ namespace ColorSpace {
 		Xyz xyz((x+y)/1.02, y, -(z-y)/0.847);
 		XyzConverter::ToColor(color, &xyz);
 	}
+	
+	void HclConverter::ToColorSpace(Rgb *color, Hcl *item) {
+	  if (!color->valid) {
+	    item->valid = false;
+	    return;
+	  }
+	  item->valid = true;
+	  Luv luv;
+	  
+	  LuvConverter::ToColorSpace(color, &luv);
+	  double l = luv.l;
+	  double c = std::sqrt(luv.u*luv.u + luv.v*luv.v);
+	  double h = std::atan2(luv.v, luv.u);
+	  
+	  h = h / M_PI * 180;
+	  if (h < 0) {
+	    h += 360;
+	  }
+	  else if (h >= 360) {
+	    h -= 360;
+	  }
+	  
+	  item->l = l;
+	  item->c = c;
+	  item->h = h;
+	}
+	void HclConverter::ToColor(Rgb *color, Hcl *item) {
+	  if (!item->valid) {
+	    color->valid = false;
+	    return;
+	  }
+	  color->valid = true;
+	  Luv luv;
+	  
+	  item->h = item->h * M_PI / 180;
+	  
+	  luv.l = item->l;
+	  luv.u = std::cos(item->h)*item->c;
+	  luv.v = std::sin(item->h)*item->c;
+	  
+	  LuvConverter::ToColor(color, &luv);
+	}
 }
 
